/*
The MIT License (MIT)

Copyright (c) 2016 British Broadcasting Corporation.
This software is provided by Lancaster University by arrangement with the BBC.

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

/**
  * A MicroBitFont is 5x5.
  * Each Row is represented by a byte in the array.
  *
  * Row Format:
  *            ================================================================
  *            | Bit 7 | Bit 6 | Bit 5 | Bit 4 | Bit 3 | Bit 2 | Bit 1 | Bit 0 |
  *            ================================================================
  *            |  N/A  |  N/A  |  N/A  | Col 1 | Col 2 | Col 3 | Col 4 | Col 5 |
  *            |  0x80 |  0x40 |  0x20 | 0x10  | 0x08  | 0x04  | 0x02  | 0x01  |
  *
  * Example: { 0x08, 0x08, 0x08, 0x0, 0x08 }
  *
  * The above will produce an exclaimation mark on the second column in form the left.
  *
  * We could compress further, but the complexity of decode would likely outweigh the gains.
  */

#ifndef MICROPY_INCLUDED_NRF_BOARD_MICROBIT_MICROBITFONT_H
#define MICROPY_INCLUDED_NRF_BOARD_MICROBIT_MICROBITFONT_H

const unsigned char font_pendolino3_5x5_pad3msb[475] = {
0x0, 0x0, 0x0, 0x0, 0x0,
0x8, 0x8, 0x8, 0x0, 0x8,
0xa, 0x4a, 0x40, 0x0, 0x0,
0xa, 0x5f, 0xea, 0x5f, 0xea,
0xe, 0xd9, 0x2e, 0xd3, 0x6e,
0x19, 0x32, 0x44, 0x89, 0x33,
0xc, 0x92, 0x4c, 0x92, 0x4d,
0x8, 0x8, 0x0, 0x0, 0x0,
0x4, 0x88, 0x8, 0x8, 0x4,
0x8, 0x4, 0x84, 0x84, 0x88,
0x0, 0xa, 0x44, 0x8a, 0x40,
0x0, 0x4, 0x8e, 0xc4, 0x80,
0x0, 0x0, 0x0, 0x4, 0x88,
0x0, 0x0, 0xe, 0xc0, 0x0,
0x0, 0x0, 0x0, 0x8, 0x0,
0x1, 0x22, 0x44, 0x88, 0x10,
0xc, 0x92, 0x52, 0x52, 0x4c,
0x4, 0x8c, 0x84, 0x84, 0x8e,
0x1c, 0x82, 0x4c, 0x90, 0x1e,
0x1e, 0xc2, 0x44, 0x92, 0x4c,
0x6, 0xca, 0x52, 0x5f, 0xe2,
0x1f, 0xf0, 0x1e, 0xc1, 0x3e,
0x2, 0x44, 0x8e, 0xd1, 0x2e,
0x1f, 0xe2, 0x44, 0x88, 0x10,
0xe, 0xd1, 0x2e, 0xd1, 0x2e,
0xe, 0xd1, 0x2e, 0xc4, 0x88,
0x0, 0x8, 0x0, 0x8, 0x0,
0x0, 0x4, 0x80, 0x4, 0x88,
0x2, 0x44, 0x88, 0x4, 0x82,
0x0, 0xe, 0xc0, 0xe, 0xc0,
0x8, 0x4, 0x82, 0x44, 0x88,
0xe, 0xd1, 0x26, 0xc0, 0x4,
0xe, 0xd1, 0x35, 0xb3, 0x6c,
0xc, 0x92, 0x5e, 0xd2, 0x52,
0x1c, 0x92, 0x5c, 0x92, 0x5c,
0xe, 0xd0, 0x10, 0x10, 0xe,
0x1c, 0x92, 0x52, 0x52, 0x5c,
0x1e, 0xd0, 0x1c, 0x90, 0x1e,
0x1e, 0xd0, 0x1c, 0x90, 0x10,
0xe, 0xd0, 0x13, 0x71, 0x2e,
0x12, 0x52, 0x5e, 0xd2, 0x52,
0x1c, 0x88, 0x8, 0x8, 0x1c,
0x1f, 0xe2, 0x42, 0x52, 0x4c,
0x12, 0x54, 0x98, 0x14, 0x92,
0x10, 0x10, 0x10, 0x10, 0x1e,
0x11, 0x3b, 0x75, 0xb1, 0x31,
0x11, 0x39, 0x35, 0xb3, 0x71,
0xc, 0x92, 0x52, 0x52, 0x4c,
0x1c, 0x92, 0x5c, 0x90, 0x10,
0xc, 0x92, 0x52, 0x4c, 0x86,
0x1c, 0x92, 0x5c, 0x92, 0x51,
0xe, 0xd0, 0xc, 0x82, 0x5c,
0x1f, 0xe4, 0x84, 0x84, 0x84,
0x12, 0x52, 0x52, 0x52, 0x4c,
0x11, 0x31, 0x31, 0x2a, 0x44,
0x11, 0x31, 0x35, 0xbb, 0x71,
0x12, 0x52, 0x4c, 0x92, 0x52,
0x11, 0x2a, 0x44, 0x84, 0x84,
0x1e, 0xc4, 0x88, 0x10, 0x1e,
0xe, 0xc8, 0x8, 0x8, 0xe,
0x10, 0x8, 0x4, 0x82, 0x41,
0xe, 0xc2, 0x42, 0x42, 0x4e,
0x4, 0x8a, 0x40, 0x0, 0x0,
0x0, 0x0, 0x0, 0x0, 0x1f,
0x8, 0x4, 0x80, 0x0, 0x0,
0x0, 0xe, 0xd2, 0x52, 0x4f,
0x10, 0x10, 0x1c, 0x92, 0x5c,
0x0, 0xe, 0xd0, 0x10, 0xe,
0x2, 0x42, 0x4e, 0xd2, 0x4e,
0xc, 0x92, 0x5c, 0x90, 0xe,
0x6, 0xc8, 0x1c, 0x88, 0x8,
0xe, 0xd2, 0x4e, 0xc2, 0x4c,
0x10, 0x10, 0x1c, 0x92, 0x52,
0x8, 0x0, 0x8, 0x8, 0x8,
0x2, 0x40, 0x2, 0x42, 0x4c,
0x10, 0x14, 0x98, 0x14, 0x92,
0x8, 0x8, 0x8, 0x8, 0x6,
0x0, 0x1b, 0x75, 0xb1, 0x31,
0x0, 0x1c, 0x92, 0x52, 0x52,
0x0, 0xc, 0x92, 0x52, 0x4c,
0x0, 0x1c, 0x92, 0x5c, 0x90,
0x0, 0xe, 0xd2, 0x4e, 0xc2,
0x0, 0xe, 0xd0, 0x10, 0x10,
0x0, 0x6, 0xc8, 0x4, 0x98,
0x8, 0x8, 0xe, 0xc8, 0x7,
0x0, 0x12, 0x52, 0x52, 0x4f,
0x0, 0x11, 0x31, 0x2a, 0x44,
0x0, 0x11, 0x31, 0x35, 0xbb,
0x0, 0x12, 0x4c, 0x8c, 0x92,
0x0, 0x11, 0x2a, 0x44, 0x98,
0x0, 0x1e, 0xc4, 0x88, 0x1e,
0x6, 0xc4, 0x8c, 0x84, 0x86,
0x8, 0x8, 0x8, 0x8, 0x8,
0x18, 0x8, 0xc, 0x88, 0x18,
0x0, 0x0, 0xc, 0x83, 0x60
};

#endif // MICROPY_INCLUDED_NRF_BOARD_MICROBIT_MICROBITFONT_H
