#define MICROPY_HW_BOARD_NAME "RGB Touch Mini"
#define MICROPY_HW_MCU_NAME "ESP32-S3-PICO-1-N8R2"
#define MICROPY_PY_NETWORK_HOSTNAME_DEFAULT "RGBTouchMini"

#define MICROPY_HW_I2C0_SCL (9)
#define MICROPY_HW_I2C0_SDA (8)

#define MICROPY_HW_SPI1_MOSI (35)
#define MICROPY_HW_SPI1_MISO (37)
#define MICROPY_HW_SPI1_SCK (36)
