#define MICROPY_HW_BOARD_NAME               "LOLIN_C3_MINI"
#define MICROPY_HW_MCU_NAME                 "ESP32-C3FH4"
#define MICROPY_PY_NETWORK_HOSTNAME_DEFAULT "mpy-c3-mini"

#define MICROPY_HW_ENABLE_SDCARD            (0)
#define MICROPY_PY_MACHINE_I2S              (0)

#define MICROPY_HW_I2C0_SCL                 (10)
#define MICROPY_HW_I2C0_SDA                 (8)

#define MICROPY_HW_SPI1_MOSI                (4)
#define MICROPY_HW_SPI1_MISO                (3)
#define MICROPY_HW_SPI1_SCK                 (2)
