#define MICROPY_HW_BOARD_NAME       "NUCLEO-F446RE"
#define MICROPY_HW_MCU_NAME         "STM32F446xx"

#define MICROPY_HW_HAS_SWITCH       (1)
#define MICROPY_HW_HAS_FLASH        (1)
#define MICROPY_HW_ENABLE_RTC       (1)
#define MICROPY_HW_ENABLE_DAC       (1)
#define MICROPY_HW_ENABLE_SERVO     (1)

// HSE is 8MHz, CPU freq set to 168MHz. Using PLLQ for USB this gives a nice
// 48 MHz clock for USB. To goto 180 MHz, I think that USB would need to be
// configured to use PLLSAI
#define MICROPY_HW_CLK_PLLM (8)
#define MICROPY_HW_CLK_PLLN (336)
#define MICROPY_HW_CLK_PLLP (RCC_PLLP_DIV2)
#define MICROPY_HW_CLK_PLLQ (7)

// UART config
#define MICROPY_HW_UART1_TX     (pin_B6)    // Arduino D10, pin 17 on CN10
#define MICROPY_HW_UART1_RX     (pin_B7)    //              pin 21 on CN7
#define MICROPY_HW_UART2_TX     (pin_A2)    // Arduino D1,  pin 35 on CN10
#define MICROPY_HW_UART2_RX     (pin_A3)    // Arduino D0,  pin 37 on CN10
#define MICROPY_HW_UART2_RTS    (pin_A1)    // Arduino A1,  pin 30 on CN7
#define MICROPY_HW_UART2_CTS    (pin_A0)    // Arduino A0,  pin 28 on CN7
#define MICROPY_HW_UART3_TX     (pin_B10)   // Arduino D6,  pin 25 on CN10
#define MICROPY_HW_UART3_RX     (pin_C5)    //              pin  6 on CN10
#define MICROPY_HW_UART3_RTS    (pin_B14)   //              pin 28 on CN10
#define MICROPY_HW_UART3_CTS    (pin_B13)   //              pin 30 on CN10
#define MICROPY_HW_UART4_TX     (pin_A0)    // Arduino A0,  pin 28 on CN7
#define MICROPY_HW_UART4_RX     (pin_A1)    // Arduino A1,  pin 30 on CN7
#define MICROPY_HW_UART6_TX     (pin_C6)    //              pin  4 on CN10
#define MICROPY_HW_UART6_RX     (pin_C7)    // Arduino D9,  pin 19 on CN10
// UART 2 connects to the STM32F103 (STLINK) on the Nucleo board
// and this is exposed as a USB Serial port.
#define MICROPY_HW_UART_REPL        PYB_UART_2
#define MICROPY_HW_UART_REPL_BAUD   115200

// I2C buses
#define MICROPY_HW_I2C1_SCL (pin_B8)        // Arduino D15, pin 3 on CN10
#define MICROPY_HW_I2C1_SDA (pin_B9)        // Arduino D14, pin 5 on CN10
#define MICROPY_HW_I2C2_SCL (pin_B10)       // Arduino D6,  pin 25 on CN10
#define MICROPY_HW_I2C2_SDA (pin_B3)        // Arduino D3,  pin 31 on CN10
#define MICROPY_HW_I2C3_SCL (pin_A8)        // Arduino D7,  pin 23 on CN10
#define MICROPY_HW_I2C3_SDA (pin_C9)        //              pin  1 on CN10

// SPI buses
#define MICROPY_HW_SPI1_NSS     (pin_A15)   //              pin 17 on CN7
#define MICROPY_HW_SPI1_SCK     (pin_A5)    // Arduino D13, pin 11 on CN10
#define MICROPY_HW_SPI1_MISO    (pin_A6)    // Arduino D12, pin 13 on CN10
#define MICROPY_HW_SPI1_MOSI    (pin_A7)    // Arduino D11, pin 15 on CN10

#define MICROPY_HW_SPI2_NSS     (pin_B12)   //              pin 16 on CN10
#define MICROPY_HW_SPI2_SCK     (pin_B13)   //              pin 30 on CN10
#define MICROPY_HW_SPI2_MISO    (pin_B14)   //              pin 28 on CN10
#define MICROPY_HW_SPI2_MOSI    (pin_B15)   //              pin 26 on CN10

#define MICROPY_HW_SPI3_NSS     (pin_A4)    // Arduino A2,  pin 32 on CN7
#define MICROPY_HW_SPI3_SCK     (pin_B3)    // Arduino D3,  pin 31 on CN10
#define MICROPY_HW_SPI3_MISO    (pin_B4)    // Arduino D5,  pin 27 on CN10
#define MICROPY_HW_SPI3_MOSI    (pin_B5)    // Arduino D4,  pin 29 on CN10

#define MICROPY_HW_SPI4_NSS     (pin_B12)   //              pin 16 on CN10
#define MICROPY_HW_SPI4_SCK     (pin_B13)   //              pin 30 on CN10
#define MICROPY_HW_SPI4_MISO    (pin_A1)    //              pin 30 on CN7
#define MICROPY_HW_SPI4_MOSI    (pin_A11)   //              pin 14 on CN10

// CAN buses
#define MICROPY_HW_CAN1_TX      (pin_B9)    //              pin 5 on CN10
#define MICROPY_HW_CAN1_RX      (pin_B8)    //              pin 3 on CN10
#define MICROPY_HW_CAN2_TX      (pin_B6)    //              pin 17 on CN10
#define MICROPY_HW_CAN2_RX      (pin_B5)    //              pin 29 on CN10

// USRSW is pulled low. Pressing the button makes the input go high.
#define MICROPY_HW_USRSW_PIN        (pin_C13)
#define MICROPY_HW_USRSW_PULL       (GPIO_NOPULL)
#define MICROPY_HW_USRSW_EXTI_MODE  (GPIO_MODE_IT_FALLING)
#define MICROPY_HW_USRSW_PRESSED    (0)

// LEDs
#define MICROPY_HW_LED1             (pin_A5) // Green LD2 LED on Nucleo
#define MICROPY_HW_LED_ON(pin)      (mp_hal_pin_high(pin))
#define MICROPY_HW_LED_OFF(pin)     (mp_hal_pin_low(pin))
