metadata(version="0.5.1")

require("re")
require("base64")
require("binascii")
require("functools")
require("string")
# require("calendar") TODO
require("abc")
require("email.errors")
require("email.header")
require("email.charset")
require("email.utils")

package("email")
