#define MICROPY_HW_BOARD_NAME "Feather M4 Express"
#define MICROPY_HW_MCU_NAME   "SAMD51J19A"

#define MICROPY_HW_XOSC32K          (1)
#define MICROPY_HW_MCU_OSC32KULP    (1)

#define MICROPY_HW_QSPIFLASH        GD25Q16C
