#define MICROPY_HW_BOARD_NAME "OMGS3"
#define MICROPY_HW_MCU_NAME "ESP32-S3-PICO-1-N8R2"
#define MICROPY_PY_NETWORK_HOSTNAME_DEFAULT "OMGS3"

#define MICROPY_HW_I2C0_SCL (9)
#define MICROPY_HW_I2C0_SDA (8)

#define MICROPY_HW_SPI1_MOSI (6)
#define MICROPY_HW_SPI1_MISO (5)
#define MICROPY_HW_SPI1_SCK (4)
