#define MICROPY_HW_BOARD_NAME "ItsyBitsy M0 Express"
#define MICROPY_HW_MCU_NAME   "SAMD21G18A"

#define MICROPY_HW_DFLL_USB_SYNC    (1)

#define MICROPY_HW_SPIFLASH         (1)
#define MICROPY_HW_SPIFLASH_ID      (5)
