#define MICROPY_HW_BOARD_NAME "Generic ESP32S2 module"
#define MICROPY_HW_MCU_NAME "ESP32S2"

#define MICROPY_PY_BLUETOOTH (0)
#define MICROPY_HW_ENABLE_SDCARD (0)

// Enable UART REPL for modules that have an external USB-UART and don't use native USB.
#define MICROPY_HW_ENABLE_UART_REPL         (1)
