#ifndef __MICROPY_INCLUDED_MICROBIT_MUSIC_H__
#define __MICROPY_INCLUDED_MICROBIT_MUSIC_H__

void microbit_music_init0(void);
void microbit_music_tick(void);

#endif // __MICROPY_INCLUDED_MICROBIT_MUSIC_H__
