/* generated configuration header file - do not edit */
#ifndef R_QSPI_CFG_H_
#define R_QSPI_CFG_H_
#ifdef __cplusplus
extern "C" {
#endif

#define QSPI_CFG_PARAM_CHECKING_ENABLE (BSP_CFG_PARAM_CHECKING_ENABLE)
#define QSPI_CFG_SUPPORT_EXTENDED_SPI_MULTI_LINE_PROGRAM  (1)

#ifdef __cplusplus
}
#endif
#endif /* R_QSPI_CFG_H_ */
