#define MICROPY_HW_BOARD_NAME    "LOLIN_S2_PICO"
#define MICROPY_HW_MCU_NAME      "ESP32-S2FN4R2"
#define MICROPY_PY_NETWORK_HOSTNAME_DEFAULT "mpy-s2-pico"

#define MICROPY_PY_BLUETOOTH     (0)
#define MICROPY_HW_ENABLE_SDCARD (0)

#define MICROPY_HW_I2C0_SCL      (9)
#define MICROPY_HW_I2C0_SDA      (8)

#define MICROPY_HW_SPI1_MOSI     (35)
#define MICROPY_HW_SPI1_MISO     (36)
#define MICROPY_HW_SPI1_SCK      (37)
