import logging

logging.debug("test - debug")  # ignored by default
logging.info("test - info")  # ignored by default
logging.warning("test - warning")
logging.error("test - error")
logging.critical("test - critical")
