#ifndef MICROPY_INCLUDED_MBEDTLS_CONFIG_BOARD_H
#define MICROPY_INCLUDED_MBEDTLS_CONFIG_BOARD_H

#define MBEDTLS_ECP_NIST_OPTIM

#include "ports/renesas-ra/mbedtls/mbedtls_config_port.h"

#endif /* MICROPY_INCLUDED_MBEDTLS_CONFIG_BOARD_H */
