#ifndef R_RTC_CFG_H_
#define R_RTC_CFG_H_
#ifdef __cplusplus
extern "C" {
#endif

#define RTC_CFG_PARAM_CHECKING_ENABLE (BSP_CFG_PARAM_CHECKING_ENABLE)
#define RTC_CFG_OPEN_SET_CLOCK_SOURCE (1)

#ifdef __cplusplus
}
#endif
#endif /* R_RTC_CFG_H_ */
