/*
 * This file is part of the MicroPython project, http://micropython.org/
 *
 * The MIT License (MIT)
 *
 * Copyright (c) 2017 Glenn Ruben Bakke
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef MUSIC_TUNES_H__
#define MUSIC_TUNES_H__

struct music_tune_dadadum_s;
struct music_tune_entertainer_s;
struct music_tune_prelude_s;
struct music_tune_ode_s;
struct music_tune_nyan_s;
struct music_tune_ringtone_s;
struct music_tune_funk_s;
struct music_tune_blues_s;
struct music_tune_birthday_s;
struct music_tune_wedding_s;
struct music_tune_funeral_s;
struct music_tune_punchline_s;
struct music_tune_python_s;
struct music_tune_baddy_s;
struct music_tune_chase_s;
struct music_tune_ba_ding_s;
struct music_tune_wawawawaa_s;
struct music_tune_jump_up_s;
struct music_tune_jump_down_s;
struct music_tune_power_up_s;
struct music_tune_power_down_s;

extern const struct music_tune_dadadadum_s   microbit_music_tune_dadadadum_obj;
extern const struct music_tune_entertainer_s microbit_music_tune_entertainer_obj;
extern const struct music_tune_prelude_s     microbit_music_tune_prelude_obj;
extern const struct music_tune_ode_s         microbit_music_tune_ode_obj;
extern const struct music_tune_nyan_s        microbit_music_tune_nyan_obj;
extern const struct music_tune_ringtone_s    microbit_music_tune_ringtone_obj;
extern const struct music_tune_funk_s        microbit_music_tune_funk_obj;
extern const struct music_tune_blues_s       microbit_music_tune_blues_obj;
extern const struct music_tune_birthday_s    microbit_music_tune_birthday_obj;
extern const struct music_tune_wedding_s     microbit_music_tune_wedding_obj;
extern const struct music_tune_funeral_s     microbit_music_tune_funeral_obj;
extern const struct music_tune_punchline_s   microbit_music_tune_punchline_obj;
extern const struct music_tune_python_s      microbit_music_tune_python_obj;
extern const struct music_tune_baddy_s       microbit_music_tune_baddy_obj;
extern const struct music_tune_chase_s       microbit_music_tune_chase_obj;
extern const struct music_tune_ba_ding_s     microbit_music_tune_ba_ding_obj;
extern const struct music_tune_wawawawaa_s   microbit_music_tune_wawawawaa_obj;
extern const struct music_tune_jump_up_s     microbit_music_tune_jump_up_obj;
extern const struct music_tune_jump_down_s   microbit_music_tune_jump_down_obj;
extern const struct music_tune_power_up_s    microbit_music_tune_power_up_obj;
extern const struct music_tune_power_down_s  microbit_music_tune_power_down_obj;

#endif // MUSIC_TUNES_H__
