#ifndef MICROPY_INCLUDED_PIC16BIT_UNISTD_H
#define MICROPY_INCLUDED_PIC16BIT_UNISTD_H

// XC16 compiler doesn't seem to have unistd.h file

#define SEEK_SET 0
#define SEEK_CUR 1

typedef int ssize_t;

#endif // MICROPY_INCLUDED_PIC16BIT_UNISTD_H
