metadata(version="3.3.4")

require("os")
require("re")
require("base64")
require("random")
require("datetime")
require("urllib.parse")
require("warnings")
require("quopri")
require("email.charset")

package("email")
