#ifndef MICROPY_INCLUDED_BTSTACK_BTSTACK_CONFIG_H
#define MICROPY_INCLUDED_BTSTACK_BTSTACK_CONFIG_H

#define HCI_OUTGOING_PRE_BUFFER_SIZE 4
#define HCI_ACL_CHUNK_SIZE_ALIGNMENT 4

#include "btstack_config_common.h"

#endif
