/*
 * This file is part of the MicroPython project, http://micropython.org/
 *
 * The MIT License (MIT)
 *
 * Copyright (c) 2013, 2014 Damien P. George
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef MICROPY_INCLUDED_STM32_PIN_STATIC_AF_H
#define MICROPY_INCLUDED_STM32_PIN_STATIC_AF_H

// For debug builds some of the macros expand to use strcmp.
#include <string.h>

#include "py/mphal.h"
#include "genhdr/pins.h"
#include "genhdr/pins_af_defs.h"

#if 0 // Enable to test if AF's are statically compiled
#define mp_hal_pin_config_alt_static(pin_obj, mode, pull, fn_type) \
    mp_hal_pin_config(pin_obj, mode, pull, fn_type(pin_obj)); \
    _Static_assert(fn_type(pin_obj) != -1, ""); \
    _Static_assert(__builtin_constant_p(fn_type(pin_obj)) == 1, "")

#else

#define mp_hal_pin_config_alt_static(pin_obj, mode, pull, fn_type) \
    mp_hal_pin_config(pin_obj, mode, pull, fn_type(pin_obj))     /* Overflow Error => alt func not found */

#define mp_hal_pin_config_alt_static_speed(pin_obj, mode, pull, speed, fn_type) \
    mp_hal_pin_config(pin_obj, mode, pull, fn_type(pin_obj));     /* Overflow Error => alt func not found */ \
    mp_hal_pin_config_speed(pin_obj, speed)

#endif

#endif // MICROPY_INCLUDED_STM32_PIN_STATIC_AF_H
