#define MICROPY_HW_BOARD_NAME    "M5Stack NanoC6"
#define MICROPY_HW_MCU_NAME      "ESP32C6"

#define MICROPY_HW_ENABLE_SDCARD (0)
#define MICROPY_PY_MACHINE_I2S   (0)

#define MICROPY_HW_I2C0_SCL      (1)
#define MICROPY_HW_I2C0_SDA      (2)
