#ifndef MICROPY_INCLUDED_DRIVERS_DHT_DHT_H
#define MICROPY_INCLUDED_DRIVERS_DHT_DHT_H

#include "py/obj.h"

MP_DECLARE_CONST_FUN_OBJ_2(dht_readinto_obj);

#endif // MICROPY_INCLUDED_DRIVERS_DHT_DHT_H
