.. comment: This file is intended for global "replace" definitions.

.. |see_cpython| replace:: See CPython documentation:

.. |see_cpython_module| replace::

    *This module implements a subset of the corresponding* :term:`CPython` *module,
    as described below. For more information, refer to the original
    CPython documentation:*
