/* This file is part of the MicroPython project, http://micropython.org/
 * The MIT License (MIT)
 * Copyright (c) 2019 Damien P. George
 */
#ifndef MICROPY_INCLUDED_STM32L1XX_HAL_CONF_H
#define MICROPY_INCLUDED_STM32L1XX_HAL_CONF_H

#include "boards/stm32l1xx_hal_conf_base.h"

#endif // MICROPY_INCLUDED_STM32L1XX_HAL_CONF_H
